﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.DragDrop;
using Telerik.Windows.DragDrop.Behaviors;

namespace AutocCompleteWPF
{
    /// <summary>
    /// Interaction logic for AutoCompleteBoxControl.xaml
    /// </summary>
    public partial class AutoCompleteBoxControl : UserControl, INotifyPropertyChanged 
    {
        private ObservableCollection<string> items;

        public ObservableCollection<string> Items
        {
            get
            {
                return this.items;
            }

            set
            {
                if (this.items != value)
                {
                    this.items = value;
                    this.OnPropertyChanged("Items");
                }
            }
        }

        public AutoCompleteBoxControl()
        {
            InitializeComponent();

            DragDropManager.AddDragInitializeHandler(this.Auto, OnDragInitialize);
            DragDropManager.AddDropHandler(this.Auto, OnDrop);
            DragDropManager.AddDragDropCompletedHandler(this.Auto, OnDragDropCompleted);
            DragDropManager.AddGiveFeedbackHandler(this.Auto, OnGiveFeedback);
        }

        private void OnGiveFeedback(object sender, Telerik.Windows.DragDrop.GiveFeedbackEventArgs e)
        {
            e.SetCursor(Cursors.Hand);
            e.UseDefaultCursors = false;
            e.Handled = true;
        }

        private void OnDragDropCompleted(object sender, DragDropCompletedEventArgs e)
        {
            var isSuccessfullyDropped = DragDropPayloadManager.GetDataFromObject(e.Data, "IsSuccessfullyDropped");
            if (isSuccessfullyDropped != null && (bool)isSuccessfullyDropped)
            {                
                var auto = sender as RadAutoCompleteBox;
                var data = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedData");
                ((IList)auto.SelectedItems).Remove(data.ToString());
                (auto.ItemsSource as ObservableCollection<string>).Remove(data.ToString());
            }

            e.Handled = true;
        }

        private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            var data = DragDropPayloadManager.GetDataFromObject(e.Data, "DraggedData");
            (e.Data as DataObject).SetData("IsSuccessfullyDropped", true);
            var auto = sender as RadAutoCompleteBox;
            (auto.ItemsSource as ObservableCollection<string>).Add(data.ToString());
            ((IList)auto.SelectedItems).Add(data.ToString());
            e.Handled = true;
        }

        private void OnDragInitialize(object sender, DragInitializeEventArgs e)
        {
            var dataItem = (e.OriginalSource as StackPanel).DataContext;
            var payload = DragDropPayloadManager.GeneratePayload(null);
            payload.SetData("DraggedData", dataItem);
            e.Data = payload;
            e.AllowedEffects = DragDropEffects.All;
            e.DragVisual = new DragVisual { Content = dataItem, ContentTemplate = this.Resources["ItemTemplate"] as DataTemplate };
            e.DragVisualOffset = new Point(e.RelativeStartPoint.X, e.RelativeStartPoint.Y + 25);
            e.Handled = true;
        }

        private void RadAutoCompleteBox_PreviewDragOver(object sender, System.Windows.DragEventArgs e)
        {
            e.Handled = true;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }
    }
}
